/*
 * Rexx/EEC test program.
 * Encodes/encrypts/compresses various strings.
 */
Call RxFuncAdd 'EECLoadFuncs', 'rexxeec', 'EECLoadFuncs'
Call EECLoadFuncs

Numeric Digits 100

Parse Version ver
Parse Source . . src

Say 'We are running Rexx/EEC version:' eecvariable( 'version' ) 'with' ver

errors = 0
str = 'This  is  the original short string.'
longstr = 'This is the original very long string that should be longer than 256 bytes to fully test the API and the memory allocation routines. If the string is longer'
longstr = longstr ' than 256, then memory is allocated for the strptr element of the RXSTRING structure. Less than 256 bytes the existing memory is used instead.'
key = 'mykey'

Parse Arg test
If test \= '' Then Interpret Signal test
/*
 * Encoding
 */
encode:
Say
Say '>>>>>Encoding...'
Call CheckEncode str, 'UU', 'uu encoding/decoding short string.'
Call CheckEncode longstr, 'UU', 'uu encoding/decoding long string.'
Call CheckEncode str, 'BASE64', 'base64 encoding/decoding short string.'
Call CheckEncode longstr, 'BASE64', 'base64 encoding/decoding long string.'
Call CheckEncode str, 'RF3986', 'RFC3986 encoding/decoding short string.'
Call CheckEncode longstr, 'RF3986', 'RFC3986 encoding/decoding long string.'
Call CheckEncode str, 'URL', 'URL encoding/decoding short string.'
Call CheckEncode longstr, 'URL', 'URL encoding/decoding long string.'
Call CheckEncode 'Special Characters: & % : ?', 'URL', 'URL encoding/decoding string with special characters.'
Call CheckEncode 'Special Characters: & is <<<&>>> " '||x2c(27), 'HTML', 'html encoding/decoding string with special characters.'
If test \= '' Then Signal endoftests
/*
 * Encrypting
 */
encrypt:
Say
Say '>>>>>Encrypting...'
Do i = 1 To Words( !REXXEEC.!ENCRYPT_TYPES )
   type = Word( !REXXEEC.!ENCRYPT_TYPES, i )
   Call CheckEncrypt str, key, type, 'encrypting/decrypting short string with' type'.'
   Call CheckEncrypt longstr, key, type, 'encrypting/decrypting long string with' type'.'
End
Say
Say 'Encryping/Decrypting 200 random string lengths up to 10000...'
Do i = 1 To 200
   num = Random( 1, 10000 )
   str = GenerateString( num )
   Do j = 1 To Words( !REXXEEC.!ENCRYPT_TYPES )
      type = Word( !REXXEEC.!ENCRYPT_TYPES, j )
      Call CheckEncrypt str, 'mykey', type, 'encrypting/decrypting string length of' num 'with' type'.'
   End
   Call Charout ,'.'
End
If test \= '' Then Signal endoftests
/*
 * Compressing
 */
compress:
Say
Say '>>>>>Compressing...'
Do i = 1 To Words( !REXXEEC.!COMPRESS_TYPES )
   type = Word( !REXXEEC.!COMPRESS_TYPES, i )
   If type = 'GZIP' Then Iterate i
   Call CheckCompress str, type, 'compressing/decompressing short string with' type '.'
   Call CheckCompress longstr, type, 'compressing/decompressing long string with' type '.'
End
Say
Say 'Compressing/Decompressing 200 random string lengths up to 10000...'
Do i = 1 To 200
   num = Random( 1, 10000 )
   str = GenerateString( num )
   Do j = 1 To Words( !REXXEEC.!COMPRESS_TYPES )
      type = Word( !REXXEEC.!COMPRESS_TYPES, j )
      If type = 'GZIP' Then Iterate j
      Call CheckCompress str, type, 'compressing/decompressing string length of' num 'with' type'.'
   End
   Call Charout ,'.'
End
Say
Say 'Not testing GZIP format'
If test \= '' Then Signal endoftests
/*
 * CRC, MD5, SHA1,...
 */
hash:
Say
Say '>>>>>SHA1 Hash...'
Call CheckSHA str, '1', '23aa65b571d9ce6b15b211141ae83d39acb08ccf', 'calculating SHA1 for short string.'
Call CheckSHA longstr, '1', '7b70335424b49364ed750f878d8526c3aa7bbde8', 'calculating SHA1 for long string.'
If Stream( src, 'C', 'QUERY EXISTS' ) \= '' Then
   Say 'SHA1 for file: "'src'" is' eecsha( src, '1', 'File' )
Say
Say '>>>>>SHA256 Hash...'
Call CheckSHA str, '256', '0de3f07d0f3916f7c73e0f726258c424b5a1957a7a5c3403891eafe870816308', 'calculating SHA256 for short string.'
Call CheckSHA longstr, '256', '7b650bcb2574c5221310d857605a6666ac4d19fe0454213165447a978dc2cfee', 'calculating SHA256 for long string.'
If Stream( src, 'C', 'QUERY EXISTS' ) \= '' Then
   Say 'SHA256 for file: "'src'" is' eecsha( src, '256', 'File' )
If test \= '' Then Signal endoftests

checksum:
Say
Say '>>>>>MD5 Checksum...'
Call CheckMD str, '5', 'ae91e0ec2bf97b1ca5d8893c19530113', 'calculating MD5 for short string.'
Call CheckMD longstr, '5', '0a92bd56c820d4749ba78dbd805adcd5', 'calculating MD5 for long string.'
If Stream( src, 'C', 'QUERY EXISTS' ) \= '' Then
   Say 'MD5 for file: "'src'" is' eecmd( src, '5', 'File' )
Say
Say '>>>>>CRC Checksum...'
Call CheckCRC str, '16', '18601', 'calculating CRC16 for short string.'
Call CheckCRC longstr, '16', '57626', 'calculating CRC16 for long string.'
Call CheckAllCRC
/*
Say 'CRC for: "123456789" is' eeccrc( '123456789', 16 )
Say 'CRC-64 for: "123456789" is' eeccrc( '123456789', 'crc-64' )
Say 'CRC-dd for: "123456789" is' eeccrc( '123456789', 'crc-dd' )
*/
If Stream( src, 'C', 'QUERY EXISTS' ) \= '' Then
   Do
      Say 'CRC (POSIX) for file: "'src'" is' eeccrc( src, 'POSIX', 'File' )
      Say 'CRC (CRC-32-BZIP2) for file: "'src'" is' eeccrc( src, 'CRC-32-BZIP2', 'File' )
   End
If test \= '' Then Signal endoftests
/*
 * TOTP
 */
totp:
Say
Say '>>>>>Time-based One-Time Password(TOTP)...'
secret = eectotp( 'SECRET' )
If secret = '' Then Say 'Error' !REXXEEC.!INTCODE '['!REXXEEC.!INTERRM']' eectotp( 'SECRET' )
Else Say 'New secret key:' secret

key = eectotp( 'KEY', secret )
If key = '' Then Say 'Error' !REXXEEC.!INTCODE '['!REXXEEC.!INTERRM']' eectotp( 'KEY', secret )
Else Say 'New OTP based on new secret key:' key
Say

ref_secret = 'BGNLSG2OU2JE56J3IP3H2LIHVCHNW6YI'
ref_time = 1608698586
ref_key = 734257
key = eectotp( 'KEY', ref_secret, ref_time )
If key = '' Then Say 'Error generating key:' !REXXEEC.!INTCODE '['!REXXEEC.!INTERRM']'
Else
   Do
      Say 'Using fixed secret:' ref_secret 'and fixed time:' ref_time
      Say 'Key generated:' key 'should be:' ref_key
   End
Say

ref_time = 0
key = eectotp( 'KEY', ref_secret, ref_time )
If key = '' Then Say 'Error' !REXXEEC.!INTCODE '['!REXXEEC.!INTERRM']' eectotp( 'KEY', ref_secret, ref_time )
Else
   Do
      Say 'If you add the fixed secret key:' ref_secret 'to Google Authenticator or Authy, the new key should match:' key
   End

valid = eectotp( 'VERIFY', ref_secret, key )
If valid = '' Then Say 'Error' !REXXEEC.!INTCODE '['!REXXEEC.!INTERRM']' eectotp( 'VALID', ref_secret, key )
Else
   Do
      Say 'Validating key:' key 'is' valid
   End
If test \= '' Then Signal endoftests
/*
 * QR Code
 */
qrcode:
Say '>>>>>QRcode...'
esc = d2c(27)
black = '##'
white = '  '
border = 2
black = esc'[40m  'esc'[0m'
white = esc'[47m  'esc'[0m'
str = 'http://www.rexx.org'
Say 'QRCode for:' str
rcode = eecqrcode( str, 'matrix.' )
If rcode = 0 Then
   Do
      -- draw top border
      Do i = 1 To border
         Say Copies( white, matrix.0 + ( 2 * border ) )
      End
      Do i = 1 To matrix.0
         -- draw left border
         Call Charout , Copies( white, border )
         Do j = 1 To matrix.0
            pixel = Substr( matrix.i, j, 1 )
            If pixel = 1 Then Call Charout , black
            Else Call Charout , white
         End
         -- draw right border
         Call Charout , Copies( white, border )
         Say
      End
      -- draw bottom border
      Do i = 1 To border
         Say Copies( white, matrix.0 + ( 2 * border ) )
      End
   End
If test \= '' Then Signal endoftests
/*
 * Luhn check character
 */
luhn:
Say '>>>>>Luhn check character...'
map = '0123456789'
str = '6473891'
Say 'Check character for' str 'with map:' map 'is:' eecluhn( 'generate', map, str )
ch = eecluhn( 'verify', map, str'7' )
If ch = '' Then ch = 'No:' !REXXEEC.!INTERRM
Say 'Does' str'7' 'have a valid check character with map:' map':' ch
ch = eecluhn( 'verify', map, str'6' )
If ch = '' Then ch = 'No:' !REXXEEC.!INTERRM
Say 'Does' str'6' 'have a valid check character with map:' map':' ch
ch = eecluhn( 'verify', map, 'Y'str )
If ch = '' Then ch = 'No:' !REXXEEC.!INTERRM
Say 'Does' 'Y'str 'have a valid check character with map:' map':' ch
ch = eecluhn( 'verify', map, str'Z' )
If ch = '' Then ch = 'No:' !REXXEEC.!INTERRM
Say 'Does' str'Z' 'have a valid check character with map:' map':' ch
If test \= '' Then Signal endoftests
/*
 * Misc
 */
misc:
Say '>>>>>Misc...'
Say 'Encoding Types supported:' !REXXEEC.!ENCODE_TYPES
Say 'Encryption Types supported:' !REXXEEC.!ENCRYPT_TYPES
Say 'Compression types supported:' !REXXEEC.!COMPRESS_TYPES
Say 'Message Digest types supported:' !REXXEEC.!MD_TYPES
Say 'Secure Hash Algorithms supported:' !REXXEEC.!SHA_TYPES
Say 'Cyclic Redunancy Check types supported:' !REXXEEC.!CRC_TYPES
Say
Say 'Setting CONSTANTPREFIX to "!TEST.!"'
Call eecvariable 'CONSTANTPREFIX', '!TEST.!'
Say 'Encoding Types supported:' !TEST.!ENCODE_TYPES
Say 'Encryption Types supported:' !TEST.!ENCRYPT_TYPES
endoftests:
Exit 0

CheckVarious: Procedure Expose !REXXEEC.
Say
Say 'Compressing/Decompressing and Encryping/Decrypting 200 random string lengths up to 10000...'
Do i = 1 To 200
   num = Random( 1, 10000 )
   str = GenerateString( num )
   Do j = 1 To Words( !REXXEEC.!ENCRYPT_TYPES )
      type = Word( !REXXEEC.!ENCRYPT_TYPES, j )
      Call CheckEncrypt str, 'mykey', type, 'encrypting/decrypting string length of' num 'with' type'.'
   End
   Do j = 1 To Words( !REXXEEC.!COMPRESS_TYPES )
      type = Word( !REXXEEC.!COMPRESS_TYPES, j )
      Call CheckCompress str, type, 'compressing/decompressing string length of' num 'with' type'.'
   End
   Call Charout ,'.'
End
Say
Return

CheckEncode: Procedure Expose !REXXEEC.
Parse Arg str, type, msg
If eecdecode( eecencode( str, type ), type ) \== str Then
   Do
      Say 'Error' !REXXEEC.!INTCODE '['!REXXEEC.!INTERRM']' msg
   End
Return

CheckEncrypt: Procedure Expose !REXXEEC.
Parse Arg str, key, type, msg
If eecdecrypt( eecencrypt( str, key, type ), key, type ) \== str Then
   Do
      Say 'Error' !REXXEEC.!INTCODE '['!REXXEEC.!INTERRM']' msg
   End
Return

CheckCompress: Procedure Expose !REXXEEC.
Parse Arg str, type, msg
comp = eeccompress( str, type )
If comp = '' Then
   Do
      Say 'Error' !REXXEEC.!INTCODE '['!REXXEEC.!INTERRM']' msg
   End
If eecdecompress( comp, type ) \== str Then
   Do
      Say 'Error' !REXXEEC.!INTCODE '['!REXXEEC.!INTERRM']' msg
   End
Return

GenerateString: Procedure
Parse Arg len
str = ''
Do i = 1 To len
   str = str || d2c( Random( 20, 122 ) )
End
Return str

CheckMD: Procedure Expose !REXXEEC.
Parse Arg str, type, md, msg
mymd = eecmd( str, type )
If mymd \= md Then
   Do
      Say 'Error calculating MD('type') for string: "'str'". MD should be: "'md'" but calculated "'mymd'"'
      If !REXXEEC.!INTERRM \= '' Then Say 'Further information:' !REXXEEC.!INTERRM
   End
Return

CheckSHA: Procedure Expose !REXXEEC.
Parse Arg str, type, sha, msg
mysha = eecsha( str, type )
If mysha \= sha Then
   Do
      Say 'Error calculating SHA('type') for string: "'str'". SHA should be: "'sha'" but calculated "'mysha'"'
      If !REXXEEC.!INTERRM \= '' Then Say 'Further information:' !REXXEEC.!INTERRM
   End
Return

GenerateString: Procedure
Parse Arg len
str = ''
Do i = 1 To len
   str = str || d2c( Random( 20, 122 ) )
End
Return str

CheckMD: Procedure Expose !REXXEEC.
Parse Arg str, type, md, msg
mymd = eecmd( str, type )
If mymd \= md Then
   Do
      Say 'Error calculating MD('type') for string: "'str'". MD should be: "'md'" but calculated "'mymd'"'
      If !REXXEEC.!INTERRM \= '' Then Say 'Further information:' !REXXEEC.!INTERRM
   End
Return

CheckCRC: Procedure Expose !REXXEEC.
Parse Arg str, type, crc, msg
mycrc = eeccrc( str, type )
If mycrc \= crc Then
   Do
      Say 'Error calculating CRC('type') for string: "'str'". CRC should be: "'crc'" but calculated "'mycrc'"'
      If !REXXEEC.!INTERRM \= '' Then Say 'Further information:' !REXXEEC.!INTERRM
   End
Return

CheckAllCRC: Procedure Expose !REXXEEC.
str = '123456789'
crctypes = '16 crc-5 crc-8 dallas-1-wire crc-12-3gpp crc-15 crc-16 crc-16-usb crc-16-modbus crc-16-genibus ccitt r-crc-16 kermit x-25 xmodem zmodem crc-24 crc-32 crc-32c crc-32-mpeg crc-32-bzip2 posix jam xfer crc-64 crc-64-jones crc-64-xz'
crcvalues = '36974 25 244 161 3503 1438 47933 46280 19255 54862 10673 126 8585 36974 12739 12739 2215682 3421780262 3808858755 58124007 4236843288 1985902208 873187033 3171672888 5090661014116757502 14602665702277640833 11051210869376104954'
Do i = 1 To Words( crctypes )
   type = Word( crctypes, i )
   crc = Word( crcvalues, i )
   mycrc = eeccrc( str, type )
   If mycrc \= crc Then
      Do
         Say 'Error calculating CRC('type') for string: "'str'". CRC should be: "'crc'" but calculated "'mycrc'"'
         If !REXXEEC.!INTERRM \= '' Then Say 'Further information:' !REXXEEC.!INTERRM
      End
End
Return
